@@ -118,29 +118,14 @@ panel.histogram <-
 
         if (nb>1)
         {
-            grid.rect(gp =
-                      gpar(fill = col, alpha = alpha,
-                           col = border, lty = lty, lwd = lwd),
-                      x = breaks[-nb],
-                      y = 0,
-                      height = y,
-                      width = diff(breaks),
-                      just = c("left", "bottom"),
-                      default.units = "native")
-
-##             for(i in seq(length =  nb-1))
-##                 if (y[i]>0)
-##                 {
-##                     grid.rect(gp =
-##                               gpar(fill = col, alpha = alpha,
-##                                    col = border, lty = lty, lwd = lwd),
-##                               x = breaks[i],
-##                               y = 0,
-##                               height = y[i],
-##                               width = breaks[i+1]-breaks[i],
-##                               just = c("left", "bottom"),
-##                               default.units = "native")
-##                 }
+            panel.rect(x = breaks[-nb],
+                       y = 0,
+                       height = y,
+                       width = diff(breaks),
+                       col = col, alpha = alpha,
+                       border = border, lty = lty,
+                       lwd = lwd,
+                       just = c("left", "bottom"))
         }
     }
 }
@@ -148,26 +133,7 @@ panel.histogram <-
 
 
 
-
-
-
-
-
-histogram <- function(x, ...)
-{
-    ocall <- match.call()
-    formula <- ocall$formula
-    if (!is.null(formula))
-    {
-        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
-        ocall$formula <- NULL
-        if (!("x" %in% names(ocall))) ocall$x <- formula else warning("'formula' overridden by 'x'")
-        eval(ocall, parent.frame())
-    }
-    else UseMethod("histogram")
-}
-
-
+histogram <- function(x, data, ...) UseMethod("histogram")
 
 
 histogram.factor <- histogram.numeric <-
@@ -180,7 +146,7 @@ histogram.factor <- histogram.numeric <-
     ccall$xlab <- xlab
     ccall$x <- ~x
     ccall[[1]] <- as.name("histogram")
-    ans <- eval(ccall, parent.frame())
+    ans <- eval.parent(ccall)
     ans$call <- ocall
     ans
 }
@@ -218,17 +184,16 @@ histogram.formula <-
              subscripts = !is.null(groups),
              subset = TRUE)
 {
-
-    ## dots <- eval(substitute(list(...)), data, parent.frame())
+    formula <- x
     dots <- list(...)
 
     ## Step 1: Evaluate x, y, etc. and do some preprocessing
 
-    groups <- eval(substitute(groups), data, parent.frame())
-    subset <- eval(substitute(subset), data, parent.frame())
+    groups <- eval(substitute(groups), data, environment(formula))
+    subset <- eval(substitute(subset), data, environment(formula))
 
     form <-
-        latticeParseFormula(x, data, subset = subset,
+        latticeParseFormula(formula, data, subset = subset,
                             groups = groups, multiple = allow.multiple,
                             outer = outer, subscripts = TRUE,
                             drop = drop.unused.levels)
@@ -247,57 +212,55 @@ histogram.formula <-
         else eval(prepanel)
 
     cond <- form$condition
-    number.of.cond <- length(cond)
     x <- form$right
-    if (number.of.cond == 0) {
+    if (length(cond) == 0)
+    {
         strip <- FALSE
         cond <- list(gl(1, length(x)))
-        number.of.cond <- 1
     }
-
-    
     if (missing(xlab)) xlab <- form$right.name
     if (missing(ylab)) ylab <- TRUE
 
     ##if(!(is.numeric(x) || is.factor(x)))
     ##    warning("x should be numeric")
     ##x <- as.numeric(x)
+
     ## create a skeleton trellis object with the
     ## less complicated components:
 
-    foo <- do.call("trellis.skeleton",
-                   c(list(cond = cond,
-                          aspect = aspect,
-                          strip = strip,
-                          panel = panel,
-                          xlab = xlab,
-                          ylab = ylab,
-                          xlab.default = form$right.name,
-                          ylab.default = "dummy"), dots))
+    foo <-
+        do.call("trellis.skeleton",
+                c(list(formula = formula, 
+                       cond = cond,
+                       aspect = aspect,
+                       strip = strip,
+                       panel = panel,
+                       xlab = xlab,
+                       ylab = ylab,
+                       xlab.default = form$right.name,
+                       ylab.default = "dummy"), dots))
                           
-
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
     foo$call <- match.call()
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
-    ## scales <- eval(substitute(scales), data, parent.frame())
     if (is.character(scales)) scales <- list(relation = scales)
     scales <- updateList(default.scales, scales)
-    foo <- c(foo,
-             do.call("construct.scales", scales))
-
+    foo <- c(foo, do.call("construct.scales", scales))
 
     ## Step 3: Decide if limits were specified in call:
     
     have.xlim <- !missing(xlim)
-    if (!is.null(foo$x.scales$limit)) {
+    if (!is.null(foo$x.scales$limit))
+    {
         have.xlim <- TRUE
         xlim <- foo$x.scales$limit
     }
     have.ylim <- !missing(ylim)
-    if (!is.null(foo$y.scales$limit)) {
+    if (!is.null(foo$y.scales$limit))
+    {
         have.ylim <- TRUE
         ylim <- foo$y.scales$limit
     }
@@ -306,7 +269,8 @@ histogram.formula <-
 
     have.xlog <- !is.logical(foo$x.scales$log) || foo$x.scales$log
     have.ylog <- !is.logical(foo$y.scales$log) || foo$y.scales$log
-    if (have.xlog) {
+    if (have.xlog)
+    {
         xlog <- foo$x.scales$log
         xbase <-
             if (is.logical(xlog)) 10
@@ -336,36 +300,23 @@ histogram.formula <-
         type <- "density"
     type <- match.arg(type)
     if (prefer.density && type != "density")
-        warning(paste("type='", type, "' can be misleading in this context",
-                      sep = ""))
+        warning(gettextf("type='%s' can be misleading in this context", type))
 
     ## this is normally done earlier (in trellis.skeleton), but in
     ## this case we needed to wait till type is determined
 
     foo$ylab.default <-
         switch(type,
-               count   = "Count",
-               percent = "Percent of Total",
-               density = "Density")
+               count   = gettext("Count"),
+               percent = gettext("Percent of Total"),
+               density = gettext("Density"))
 
     ## Step 5: Process cond
 
     cond.max.level <- unlist(lapply(cond, nlevels))
 
-    ## old NA-handling
-##     id.na <- is.na(x)
-##     for (var in cond)
-##         id.na <- id.na | is.na(var)
-##     if (!any(!id.na)) stop("nothing to draw")
+    ## Step 6: Determine packets
 
-    ## new NA-handling: will retain NA's in x
-
-    id.na <- do.call("pmax", lapply(cond, is.na))
-    if (!any(!id.na)) stop("nothing to draw")
-
-    ## Step 6: Evaluate layout, panel.args.common and panel.args
-
-    ## equal.widths <- eval(equal.widths, data, parent.frame()) #keep this way ?
     foo$panel.args.common <-
         c(list(breaks = breaks,
                type = type,
@@ -374,31 +325,28 @@ histogram.formula <-
           dots)
     if (subscripts) foo$panel.args.common$groups <- groups
 
-    nplots <- prod(cond.max.level)
-    if (nplots != prod(sapply(foo$condlevels, length))) stop("mismatch")
-    foo$panel.args <- vector(mode = "list", length = nplots)
+    npackets <- prod(cond.max.level)
+    if (npackets != prod(sapply(foo$condlevels, length))) 
+        stop("mismatch in number of packets")
+    foo$panel.args <- vector(mode = "list", length = npackets)
 
-    cond.current.level <- rep(1, number.of.cond)
-
-    for (panel.number in seq(length = nplots))
+    foo$packet.sizes <- numeric(npackets)
+    if (npackets > 1)
     {
-        id <- !id.na
-        for(i in 1:number.of.cond)
-        {
-            var <- cond[[i]]
-            id <- id &
-            if (is.shingle(var))
-                ((var >=
-                  levels(var)[[cond.current.level[i]]][1])
-                 & (var <=
-                    levels(var)[[cond.current.level[i]]][2]))
-            else (as.numeric(var) == cond.current.level[i])
-        }
+        dim(foo$packet.sizes) <- sapply(foo$condlevels, length)
+        dimnames(foo$packet.sizes) <- lapply(foo$condlevels, as.character)
+    }
 
-        foo$panel.args[[panel.number]] <-
-            list(x = x[id])
+    cond.current.level <- rep(1, length(cond))
+
+    for (packet.number in seq(length = npackets))
+    {
+        id <- compute.packet(cond, cond.current.level)
+        foo$packet.sizes[packet.number] <- sum(id)
+
+        foo$panel.args[[packet.number]] <- list(x = x[id])
         if (subscripts)
-            foo$panel.args[[panel.number]]$subscripts <-
+            foo$panel.args[[packet.number]]$subscripts <-
                 subscr[id]
 
         cond.current.level <-
@@ -415,13 +363,11 @@ histogram.formula <-
                                      panel.args.common = foo$panel.args.common,
                                      panel.args = foo$panel.args,
                                      aspect = aspect,
-                                     nplots = nplots,
+                                     npackets = npackets,
                                      x.axs = foo$x.scales$axs,
                                      y.axs = foo$y.scales$axs),
                    cond.orders(foo))
     foo[names(more.comp)] <- more.comp
-    
-
 
 
     if (is.null(foo$legend) && !is.null(groups) &&
