@@ -38,14 +38,16 @@ getLabelList <- function(label, text.settings, default.label = NULL)
     if (!is.null(label))
     {
         if (inherits(label, "grob")) return(label)
-        ans <- list(label = 
-                    if (is.characterOrExpression(label)) label
-                    else if (is.list(label) && (is.null(names(label)) || names(label)[1] == "")) label[[1]]
-                    else default.label,
-                    col = text.settings$col, cex = text.settings$cex,
-                    fontfamily = text.settings$fontfamily,
-                    fontface = text.settings$fontface,
-                    font = text.settings$font)
+        ans <-
+            list(label = 
+                 if (is.characterOrExpression(label)) label
+                 else if (is.list(label) && (is.null(names(label)) || names(label)[1] == "")) label[[1]]
+                 else default.label,
+                 col = text.settings$col, cex = text.settings$cex,
+                 fontfamily = text.settings$fontfamily,
+                 fontface = text.settings$fontface,
+                 font = text.settings$font,
+                 lineheight = text.settings$lineheight)
         if (is.list(label)) ans[names(label)] <- label
     }
     else ans <- NULL
@@ -61,14 +63,14 @@ getLabelList <- function(label, text.settings, default.label = NULL)
 
 grobFromLabelList <- function(lab, name = "label", rot = 0)
 {
-    if (is.null(lab)) return (NULL)
+    if (is.null(lab) || (is.character(lab) && lab == "")) return (NULL)
     if (inherits(lab, "grob")) return(lab)
-
     textGrob(label = lab$label, name= name, rot = rot,
              gp =
              gpar(col = lab$col,
                   fontfamily = lab$fontfamily,
                   fontface = chooseFace(lab$fontface, lab$font),
+                  lineheight = lab$lineheight,
                   cex = lab$cex))
 }
 
@@ -161,11 +163,14 @@ print.trellis <-
     lattice.setStatus(current.prefix = prefix)
     lattice.setStatus(plot.index = 1 + lattice.getStatus("plot.index"))
 
-    fontsize.text <- trellis.par.get("fontsize")$text
+    global.gpar <-
+        do.call(gpar,
+                updateList(trellis.par.get("grid.pars"),
+                           list(fontsize = trellis.par.get("fontsize")$text)))
 
     if (!missing(position))
     {
-        if (length(position)!=4) stop("Incorrect value of position")
+        stopifnot (length(position) == 4)
         if (new)
         {
             grid.newpage()
@@ -179,7 +184,7 @@ print.trellis <-
 
         if (!missing(split))
         {
-            if (length(split)!=4) stop("Incorrect value of split")
+            stopifnot (length(split) == 4)
             pushViewport(viewport(layout = grid.layout(nrow = split[4], ncol = split[3]),
                                   name = trellis.vpname("split") ))
             pushViewport(viewport(layout.pos.row = split[2], layout.pos.col = split[1],
@@ -190,7 +195,7 @@ print.trellis <-
 
     else if (!missing(split))
     {
-        if (length(split)!=4) stop("Incorrect value of split")
+        stopifnot(length(split) == 4)
         if (new)
         {
             grid.newpage()
@@ -248,6 +253,10 @@ print.trellis <-
         if (is.function(x$strip)) x$strip 
         else if (is.character(x$strip)) get(x$strip)
         else eval(x$strip)
+    strip.left <- 
+        if (is.function(x$strip.left)) x$strip.left 
+        else if (is.character(x$strip.left)) get(x$strip.left)
+        else eval(x$strip.left)
 
     axis.line <- trellis.par.get("axis.line")
     axis.text <- trellis.par.get("axis.text")
@@ -262,9 +271,6 @@ print.trellis <-
         panel.height[[1]] <- x$aspect.ratio * panel.width[[1]]
 
 
-
-
-
     ## Evaluate the legend / key grob(s): 
 
     legend <- evaluate.legend(x$legend)
@@ -348,12 +354,14 @@ print.trellis <-
 
 
     strip.col.default.bg <-
-        rep(trellis.par.get("strip.background")$col, length=number.of.cond)
+        rep(trellis.par.get("strip.background")$col,
+            length = number.of.cond)
     strip.col.default.fg <-
-        rep(trellis.par.get("strip.shingle")$col,length=number.of.cond)
+        rep(trellis.par.get("strip.shingle")$col,
+            length = number.of.cond)
     strip.border <-
         lapply(trellis.par.get("strip.border"),
-               function(x) rep(x, length=number.of.cond))
+               function(x) rep(x, length = number.of.cond))
 
 
     ## Start layout calculations when only number of panels per page
@@ -385,8 +393,9 @@ print.trellis <-
     rows.per.page <- panel.layout[2]
     number.of.pages <- panel.layout[3]
     lattice.setStatus(current.plot.multipage = number.of.pages > 1)
-    ## this will also eventually be a 'status' variable
+    ## these will also eventually be 'status' variables
     current.panel.positions <- matrix(0, rows.per.page, cols.per.page)
+    current.packet.positions <- matrix(0, rows.per.page, cols.per.page)
 
 
     skip <- rep(x$skip, length = number.of.pages * rows.per.page * cols.per.page)
@@ -463,16 +472,22 @@ print.trellis <-
     ##   conditioning variables for the current panel.
 
     
-    panel.counter <- 0
+    panel.number <- 0
 
-    ## panel.counter used as an optional argument to the panel
-    ## function. Sequential counter keeping track of which panel is
-    ## being drawn
+    ## panel.number is supplied as an optional argument to the panel
+    ## function.  It's a strictly increasing sequential counter
+    ## keeping track of which panel is being drawn.  This is usually
+    ## the same as, but potentially different from the packet.number,
+    ## which is an index to which packet combination is being used.
 
-    for(page.number in 1:number.of.pages)
+    ## Here's the complication.  panel.number used to be called
+    ## panel.counter, and packet.number used to be called panel.number
+
+
+    for(page.number in seq(length = number.of.pages))
     {
-        if (!any(cond.max.level - cond.current.level < 0)) {
-            
+        if (!any(cond.max.level - cond.current.level < 0))
+        {
             if (usual)
             {
                 if (new) grid.newpage()
@@ -481,53 +496,9 @@ print.trellis <-
             }
 
             pushViewport(viewport(layout = page.layout,
-                                  gp =
-                                  gpar(fontsize = fontsize.text),
+                                  gp = global.gpar,
                                   name = trellis.vpname("toplevel")))
 
-
-
-#             if (!is.null(main))
-#                 grid.text(label = main$label, name= "main",
-#                           gp =
-#                           gpar(col = main$col,
-#                                fontfamily = main$fontfamily,
-#                                fontface = chooseFace(main$fontface, main$font),
-#                                cex = main$cex),
-#                           vp = viewport(layout.pos.row = pos.heights$main, name= "main.vp"))
-
-#             if (!is.null(sub))
-#                 grid.text(label = sub$label, name= "sub",
-#                           gp =
-#                           gpar(col = sub$col,
-#                                fontfamily = sub$fontfamily,
-#                                fontface = chooseFace(sub$fontface, sub$font),
-#                                cex = sub$cex),
-#                           vp = viewport(layout.pos.row = pos.heights$sub, name= "sub.vp"))
-
-#             if (!is.null(xlab))
-#                 grid.text(label = xlab$label, name= "xlab",
-#                           gp =
-#                           gpar(col = xlab$col,
-#                                fontfamily = xlab$fontfamily,
-#                                fontface = chooseFace(xlab$fontface, xlab$font),
-#                                cex = xlab$cex), 
-#                           vp = viewport(layout.pos.row = pos.heights$xlab,
-#                                         layout.pos.col = pos.widths$panel, name= "xlab.vp" ))
-
-#             if (!is.null(ylab))
-#                 grid.text(label = ylab$label, rot = 90, name= "ylab",
-#                           gp =
-#                           gpar(col = ylab$col,
-#                                fontfamily = ylab$fontfamily,
-#                                fontface = chooseFace(ylab$fontface, ylab$font),
-#                                cex = ylab$cex),
-#                           vp = viewport(layout.pos.col = pos.widths$ylab,
-#                                         layout.pos.row = pos.heights$panel, name= "ylab.vp"))
-
-
-
-
             if (!is.null(main))
             {
                 pushViewport(viewport(layout.pos.row = pos.heights$main,
@@ -570,13 +541,13 @@ print.trellis <-
                         !skip[(page.number-1) * rows.per.page * cols.per.page +
                               (row-1) * cols.per.page + column] )
                     {
-                        ##panel.number should be same as order.cond[cond.current.level]
-                        ##                                          ^^^^^^^^^^^^^^^^^^
-                        ##                                          (length not fixed)
+                        ## packet.number should be same as order.cond[cond.current.level]
+                        ##                                            ^^^^^^^^^^^^^^^^^^
+                        ##                                            (length not fixed)
 
-                        panel.number <- 
+                        packet.number <- 
                             do.call("[", c(list(x = order.cond), as.list(cond.current.level)))
-                        current.panel.positions[row, column] <- panel.number
+                        current.packet.positions[row, column] <- packet.number
 
                         ## this index retrieves the appropriate entry
                         ## of panel.args and [xy].limits. It has to be
@@ -587,7 +558,8 @@ print.trellis <-
                         ## incremental counter that may be used as a
                         ## panel function argument
 
-                        panel.counter <- panel.counter + 1
+                        panel.number <- panel.number + 1
+                        current.panel.positions[row, column] <- panel.number
 
                         ## this gives the row position from the bottom
                         actual.row <- if (x$as.table)
@@ -599,21 +571,21 @@ print.trellis <-
                         xlabelinfo <-
                             calculateAxisComponents(x =
                                                     if (x.relation.same) x$x.limits
-                                                    else x$x.limits[[panel.number]],
+                                                    else x$x.limits[[packet.number]],
 
                                                     at = if (is.list(x$x.scales$at))
-                                                    x$x.scales$at[[panel.number]]
+                                                    x$x.scales$at[[packet.number]]
                                                     else x$x.scales$at,
 
                                                     used.at = if (!x.relation.same)
-                                                    x$x.used.at[[panel.number]] else x$x.used.at,
+                                                    x$x.used.at[[packet.number]] else x$x.used.at,
 
                                                     num.limit = if (!x.relation.same)
-                                                    x$x.num.limit[[panel.number]] else x$x.num.limit,
+                                                    x$x.num.limit[[packet.number]] else x$x.num.limit,
 
                                                     labels =
                                                     if (is.list(x$x.scales$lab))
-                                                    x$x.scales$lab[[panel.number]]
+                                                    x$x.scales$lab[[packet.number]]
                                                     else x$x.scales$lab,
 
                                                     logsc = x$x.scales$log,
@@ -625,21 +597,21 @@ print.trellis <-
                         ylabelinfo <-
                             calculateAxisComponents(x =
                                                     if (y.relation.same) x$y.limits
-                                                    else x$y.limits[[panel.number]],
+                                                    else x$y.limits[[packet.number]],
 
                                                     at = if (is.list(x$y.scales$at))
-                                                    x$y.scales$at[[panel.number]]
+                                                    x$y.scales$at[[packet.number]]
                                                     else x$y.scales$at,
 
                                                     used.at = if (!y.relation.same)
-                                                    x$y.used.at[[panel.number]] else x$y.used.at,
+                                                    x$y.used.at[[packet.number]] else x$y.used.at,
 
                                                     num.limit = if (!y.relation.same)
-                                                    x$y.num.limit[[panel.number]] else x$y.num.limit,
+                                                    x$y.num.limit[[packet.number]] else x$y.num.limit,
 
                                                     labels =
                                                     if (is.list(x$y.scales$lab))
-                                                    x$y.scales$lab[[panel.number]]
+                                                    x$y.scales$lab[[packet.number]]
                                                     else x$y.scales$lab,
 
                                                     logsc = x$y.scales$log,
@@ -702,6 +674,44 @@ print.trellis <-
                         }
                         upViewport()
 
+                        pushViewport(viewport(layout.pos.row = pos.row,
+                                              layout.pos.col = pos.col - 1,
+                                              yscale = yscale,
+                                              clip = "off",
+                                              name =
+                                              trellis.vpname("strip",
+                                                             column = column,
+                                                             row = row,
+                                                             clip.off = TRUE)))
+
+                        ## Y-axis to the left
+                        if (x$y.scales$draw && (!y.relation.same || column == 1))
+                        {
+                            axstck <- x$y.scales$tck
+                            panel.axis(side = "left",
+                                       at = ylabelinfo$at,
+                                       labels = ylabelinfo$lab,
+                                       draw.labels = (!y.relation.same ||
+                                                      y.alternating[actual.row]==1 ||
+                                                      y.alternating[actual.row]==3), 
+                                       check.overlap = ylabelinfo$check.overlap,
+                                       outside = TRUE,
+                                       tick = TRUE,
+                                       tck = axstck[1],
+                                       rot = yaxis.rot[1],
+                                       text.col = yaxis.col.text,
+                                       text.alpha = yaxis.alpha.text,
+                                       text.cex = yaxis.cex[1],
+                                       text.font = yaxis.font,
+                                       text.fontfamily = yaxis.fontfamily,
+                                       text.fontface = xaxis.fontface,
+                                       line.col = yaxis.col.line,
+                                       line.lty = yaxis.lty,
+                                       line.lwd = yaxis.lwd,
+                                       line.alpha = yaxis.alpha.line)
+                        }
+                        upViewport()
+
 
                         pushViewport(viewport(layout.pos.row = pos.row,
                                               layout.pos.col = pos.col,
@@ -747,32 +757,6 @@ print.trellis <-
 
                         ## Y-axis
                         
-                        ## Y-axis to the left
-                        if (x$y.scales$draw && (!y.relation.same || column == 1))
-                        {
-                            axstck <- x$y.scales$tck
-                            panel.axis(side = "left",
-                                       at = ylabelinfo$at,
-                                       labels = ylabelinfo$lab,
-                                       draw.labels = (!y.relation.same ||
-                                                      y.alternating[actual.row]==1 ||
-                                                      y.alternating[actual.row]==3), 
-                                       check.overlap = ylabelinfo$check.overlap,
-                                       outside = TRUE,
-                                       tick = TRUE,
-                                       tck = axstck[1],
-                                       rot = yaxis.rot[1],
-                                       text.col = yaxis.col.text,
-                                       text.alpha = yaxis.alpha.text,
-                                       text.cex = yaxis.cex[1],
-                                       text.font = yaxis.font,
-                                       text.fontfamily = yaxis.fontfamily,
-                                       text.fontface = xaxis.fontface,
-                                       line.col = yaxis.col.line,
-                                       line.lty = yaxis.lty,
-                                       line.lwd = yaxis.lwd,
-                                       line.alpha = yaxis.alpha.line)
-                        }
 
                         ## Y-axis to the right
 
@@ -785,7 +769,7 @@ print.trellis <-
                         ## last panel.
 
                         if (x$y.scales$draw && y.relation.same &&
-                            (column == cols.per.page || panel.counter == last.panel))
+                            (column == cols.per.page || panel.number == last.panel))
                         {
                             axstck <- x$y.scales$tck
                             panel.axis(side = "right",
@@ -830,10 +814,10 @@ print.trellis <-
                                                              clip.off = FALSE)))
 
 
-                        pargs <- c(x$panel.args[[panel.number]],
+                        pargs <- c(x$panel.args[[packet.number]],
                                    x$panel.args.common,
-                                   list(panel.number = panel.number,
-                                        panel.counter = panel.counter))
+                                   list(packet.number = packet.number,
+                                        panel.number = panel.number))
 
                         if (!("..." %in% names(formals(panel))))
                             pargs <- pargs[names(formals(panel))]
@@ -859,9 +843,6 @@ print.trellis <-
 
 
 
-
-
-
 #########################################
 ###        draw strip(s)              ###
 #########################################
@@ -883,7 +864,7 @@ print.trellis <-
                                                                  column = column,
                                                                  row = row,
                                                                  clip.off = FALSE)))
-
+                            
 
                             for(i in seq(length = number.of.cond))
                             {
@@ -910,19 +891,66 @@ print.trellis <-
                             }
                             upViewport()
                         }
+            
 
-                        
-                        cond.current.level <- cupdate(cond.current.level,
-                                                      cond.max.level)
+#########################################
+###        draw strip(s) on left      ###
+#########################################
 
+
+                        if (!is.logical(strip.left)) # logical <==> FALSE
+                        {
+                            ## which.panel in original cond variables order
+                            which.panel <- cond.current.level[x$perm.cond]
+
+                            ## need to pass each index in original terms
+                            for (i in seq(along = which.panel))
+                                which.panel[i] <- x$index.cond[[i]][which.panel[i]]
+
+                            pushViewport(viewport(layout.pos.row = pos.row,
+                                                  layout.pos.col = pos.col - 1,
+                                                  clip = trellis.par.get("clip")$strip,
+                                                  name =
+                                                  trellis.vpname("strip.left",
+                                                                 column = column,
+                                                                 row = row,
+                                                                 clip.off = FALSE)))
+
+
+                            for(i in seq(length = number.of.cond))
+                            {
+
+                                ## Here, by which.given, I mean which
+                                ## in the original packet order, not
+                                ## the permuted order
+
+                                strip.left(which.given = x$perm.cond[i],
+                                           which.panel = which.panel,
+
+                                           var.name = names(x$condlevels),
+
+                                           factor.levels = if (!is.list(x$condlevels[[x$perm.cond[i]]]))
+                                           x$condlevels[[x$perm.cond[i]]] else NULL,
+
+                                           shingle.intervals = if (is.list(x$condlevels[[x$perm.cond[i]]]))
+                                           do.call("rbind", x$condlevels[[x$perm.cond[i]]]) else NULL,
+
+                                           bg = strip.col.default.bg[i],
+                                           fg = strip.col.default.fg[i],
+                                           par.strip.text = par.strip.text)
+                                
+                            }
+                            upViewport()
+                        }
+                    
+                        cond.current.level <-
+                            cupdate(cond.current.level,
+                                    cond.max.level)
                     }
                 }
 
 
 
-
-
-
             ## legend / key plotting
 
             if (!is.null(legend))
@@ -952,7 +980,7 @@ print.trellis <-
                     else if (key.space == "top")
                     {
                         pushViewport(viewport(layout.pos.row = pos.heights$key.top,
-                                              layout.pos.col = pos.widths$panel,
+                                              layout.pos.col = range(pos.widths$panel, pos.widths$strip.left),
                                               name = trellis.vpname("legend", side = "top")))
                         grid.draw(key.gf)
                         upViewport()
@@ -960,7 +988,7 @@ print.trellis <-
                     else if (key.space == "bottom")
                     {
                         pushViewport(viewport(layout.pos.row = pos.heights$key.bottom,
-                                              layout.pos.col = pos.widths$panel,
+                                              layout.pos.col = range(pos.widths$panel, pos.widths$strip.left),
                                               name = trellis.vpname("legend", side = "bottom")))
                         grid.draw(key.gf)
                         upViewport()
@@ -974,16 +1002,14 @@ print.trellis <-
                         key.corner <-
                             if (is.null(legend[[i]]$corner)) c(0,1)
                             else legend[[i]]$corner
-
                         key.x <- 
                             if (is.null(legend[[i]]$x)) key.corner[1]
                             else legend[[i]]$x
-
                         key.y <- 
                             if (is.null(legend[[i]]$y)) key.corner[2]
                             else legend[[i]]$y
-                        
-                        if (all(key.corner == c(0,1))) {
+                        if (all(key.corner == c(0,1)))
+                        {
                             pushViewport(viewport(layout = grid.layout(nrow = 3, ncol = 3,
                                                   widths = unit(c(key.x, 1, 1),
                                                   c("npc", "grobwidth", "null"),
@@ -992,7 +1018,8 @@ print.trellis <-
                                                   c("npc", "grobheight", "null"),
                                                   list(1, key.gf, 1)))))
                         }
-                        else if (all(key.corner == c(1,1))) {
+                        else if (all(key.corner == c(1,1)))
+                        {
                             pushViewport(viewport(layout = grid.layout(nrow = 3, ncol = 3,
                                                   heights = unit(c(1 - key.y, 1, 1),
                                                   c("npc", "grobheight", "null"),
@@ -1001,7 +1028,8 @@ print.trellis <-
                                                   c("null", "grobwidth", "npc"),
                                                   list(1, key.gf, 1)))))
                         }
-                        else if (all(key.corner == c(0,0))) {
+                        else if (all(key.corner == c(0,0)))
+                        {
                             pushViewport(viewport(layout = grid.layout(nrow = 3, ncol = 3,
                                                   widths = unit(c(key.x, 1, 1),
                                                   c("npc", "grobwidth", "null"),
@@ -1010,7 +1038,8 @@ print.trellis <-
                                                   c("null", "grobheight", "npc"),
                                                   list(1, key.gf, 1)))))
                         }
-                        else if (all(key.corner == c(1,0))) {
+                        else if (all(key.corner == c(1,0)))
+                        {
                             pushViewport(viewport(layout=grid.layout(nrow = 3, ncol = 3,
                                                   widths = unit(c(1, 1, 1 - key.x),
                                                   c("null", "grobwidth", "npc"),
@@ -1025,7 +1054,6 @@ print.trellis <-
                     }
                 }
             }
-            
             pushViewport(viewport(layout.pos.row = c(1, n.row),
                                   layout.pos.col = c(1, n.col),
                                   name = trellis.vpname("page")))
@@ -1034,14 +1062,17 @@ print.trellis <-
             upViewport()
         }
     }
-    if (!missing(position)) {
-        if (!missing(split)) {
+    if (!missing(position))
+    {
+        if (!missing(split))
+        {
             upViewport()
             upViewport()
         }
         upViewport()
     }
-    else if (!missing(split)) {
+    else if (!missing(split))
+    {
         upViewport()
         upViewport()
     }
@@ -1061,6 +1092,7 @@ print.trellis <-
     else
         lattice.setStatus(current.plot.saved = FALSE)
     lattice.setStatus(current.panel.positions = current.panel.positions)
+    lattice.setStatus(current.packet.positions = current.packet.positions)
     lattice.setStatus(current.focus.row = 0,
                       current.focus.column = 0,
                       vp.highlighted = FALSE,
